@@ -24,33 +24,44 @@
 #include "gnome-cmd-dialog.h"
 #include "gnome-cmd-string-dialog.h"
 #include "libgcmd-widget-factory.h"
+#include "libgcmd-utils.h"
 
-struct GnomeCmdStringDialog::Private
+struct GnomeCmdStringDialogPrivate
 {
     GnomeCmdStringDialogCallback ok_cb;
-    GFunc cancel_cb;
+    GnomeCmdCallback<GtkButton*> cancel_cb;
     gpointer data;
     gchar *error_desc;
 };
 
 
-static GnomeCmdDialogClass *parent_class = NULL;
+G_DEFINE_TYPE_WITH_PRIVATE (GnomeCmdStringDialog, gnome_cmd_string_dialog, GNOME_CMD_TYPE_DIALOG)
+
+
+static void show_error_dialog (GtkWindow *parent_window, const gchar *msg)
+{
+    GtkWidget *dialog = gtk_message_dialog_new (parent_window, GTK_DIALOG_MODAL, GTK_MESSAGE_ERROR, GTK_BUTTONS_OK, "%s", msg);
+    g_signal_connect (dialog, "response", G_CALLBACK (gtk_window_destroy), dialog);
+    gtk_window_present (GTK_WINDOW (dialog));
+}
 
 
 static void on_ok (GtkButton *button, GnomeCmdStringDialog *dialog)
 {
+    auto priv = static_cast<GnomeCmdStringDialogPrivate*> (gnome_cmd_string_dialog_get_instance_private (dialog));
+
     gboolean valid = TRUE;
 
-    if (dialog->priv->ok_cb)
+    if (priv->ok_cb)
     {
-        gchar **values = (gchar**) g_new (gpointer, dialog->rows);
+        const gchar **values = (const gchar**) g_new (gpointer, dialog->rows);
 
         for (gint i=0; i<dialog->rows; i++)
-            values[i] = (gchar*)gtk_entry_get_text (GTK_ENTRY (dialog->entries[i]));
+            values[i] = gtk_editable_get_text (GTK_EDITABLE (dialog->entries[i]));
 
-        valid = dialog->priv->ok_cb (dialog, (const gchar**)values, dialog->priv->data);
+        valid = priv->ok_cb (dialog, (const gchar**)values, priv->data);
         if (!valid)
-            create_error_dialog ("%s", dialog->priv->error_desc);
+            show_error_dialog (GTK_WINDOW (dialog), priv->error_desc);
         g_free (values);
     }
 
@@ -61,8 +72,10 @@ static void on_ok (GtkButton *button, GnomeCmdStringDialog *dialog)
 
 static void on_cancel (GtkButton *button, GnomeCmdStringDialog *dialog)
 {
-    if (dialog->priv->cancel_cb)
-        dialog->priv->cancel_cb (button, dialog->priv->data);
+    auto priv = static_cast<GnomeCmdStringDialogPrivate*> (gnome_cmd_string_dialog_get_instance_private (dialog));
+
+    if (priv->cancel_cb)
+        priv->cancel_cb (button, priv->data);
 
     gtk_widget_hide (GTK_WIDGET (dialog));
 }
@@ -72,75 +85,58 @@ static void on_cancel (GtkButton *button, GnomeCmdStringDialog *dialog)
  * Gtk class implementation
  *******************************/
 
-static void destroy (GtkObject *object)
+static void dispose (GObject *object)
 {
     GnomeCmdStringDialog *dialog = GNOME_CMD_STRING_DIALOG (object);
+    auto priv = static_cast<GnomeCmdStringDialogPrivate*> (gnome_cmd_string_dialog_get_instance_private (dialog));
 
-    if (GTK_OBJECT_CLASS (parent_class)->destroy)
-        (*GTK_OBJECT_CLASS (parent_class)->destroy) (object);
+    g_clear_pointer (&priv->error_desc, g_free);
 
-    if (dialog->priv)
-        g_free (dialog->priv->error_desc);
-
-    g_free (dialog->priv);
-    dialog->priv = NULL;
+    G_OBJECT_CLASS (gnome_cmd_string_dialog_parent_class)->dispose (object);
 }
 
 
-static void map (GtkWidget *widget)
+static void gnome_cmd_string_dialog_class_init (GnomeCmdStringDialogClass *klass)
 {
-    if (GTK_WIDGET_CLASS (parent_class)->map != NULL)
-        GTK_WIDGET_CLASS (parent_class)->map (widget);
+    G_OBJECT_CLASS (klass)->dispose = dispose;
 }
 
 
-static void class_init (GnomeCmdStringDialogClass *klass)
+static void gnome_cmd_string_dialog_init (GnomeCmdStringDialog *string_dialog)
 {
-    GtkObjectClass *object_class;
-    GtkWidgetClass *widget_class;
-
-    object_class = GTK_OBJECT_CLASS (klass);
-    widget_class = GTK_WIDGET_CLASS (klass);
-
-    parent_class = (GnomeCmdDialogClass *) gtk_type_class (GNOME_CMD_TYPE_DIALOG);
-    object_class->destroy = destroy;
-    widget_class->map = map;
-}
-
-
-static void init (GnomeCmdStringDialog *string_dialog)
-{
-    string_dialog->priv = g_new0 (GnomeCmdStringDialog::Private, 1);
     string_dialog->rows = -1;
 }
 
 
 inline void setup_widget (GnomeCmdStringDialog *string_dialog, gint rows)
 {
+    auto priv = static_cast<GnomeCmdStringDialogPrivate*> (gnome_cmd_string_dialog_get_instance_private (string_dialog));
+
     GtkWidget *dialog = GTK_WIDGET (string_dialog);
-    GtkWidget *table;
+    GtkWidget *grid;
     GtkWidget *btn;
 
     string_dialog->rows = rows;
     string_dialog->labels = (GtkWidget**) g_new (gpointer, rows);
     string_dialog->entries = (GtkWidget**) g_new (gpointer, rows);
-    string_dialog->priv->error_desc = g_strdup (_("No error description available"));
+    priv->error_desc = g_strdup (_("No error description available"));
 
-    table = create_table (dialog, rows, 2);
-    gnome_cmd_dialog_add_category (GNOME_CMD_DIALOG (dialog), table);
+    grid = create_grid (dialog);
+    gnome_cmd_dialog_add_category (GNOME_CMD_DIALOG (dialog), grid);
 
     for (gint i=0; i<rows; i++)
     {
         string_dialog->labels[i] = create_label (dialog, "");
-        table_add (table, string_dialog->labels[i], 0, i, GTK_FILL);
+        gtk_grid_attach (GTK_GRID (grid), string_dialog->labels[i], 0, i, 1, 1);
 
         string_dialog->entries[i] = create_entry (dialog, "entry", "");
         gtk_entry_set_activates_default (GTK_ENTRY (string_dialog->entries[i]), TRUE);
-        table_add (table, string_dialog->entries[i], 1, i, GtkAttachOptions (GTK_FILL|GTK_EXPAND));
+        gtk_widget_set_hexpand (string_dialog->entries[i], TRUE);
+        gtk_grid_attach (GTK_GRID (grid), string_dialog->entries[i], 1, i, 1, 1);
     }
 
-    gnome_cmd_dialog_add_button (GNOME_CMD_DIALOG (dialog), GTK_STOCK_CANCEL, GTK_SIGNAL_FUNC (on_cancel), string_dialog);
-    btn = gnome_cmd_dialog_add_button (GNOME_CMD_DIALOG (dialog), GTK_STOCK_OK, GTK_SIGNAL_FUNC (on_ok), string_dialog);
+    gnome_cmd_dialog_add_button (GNOME_CMD_DIALOG (dialog), _("_Cancel"), G_CALLBACK (on_cancel), string_dialog);
+    btn = gnome_cmd_dialog_add_button (GNOME_CMD_DIALOG (dialog), _("_OK"), G_CALLBACK (on_ok), string_dialog);
 
     gtk_widget_grab_focus (string_dialog->entries[0]);
     gtk_widget_grab_default (btn);
@@ -151,31 +147,7 @@ inline void setup_widget (GnomeCmdStringDialog *string_dialog, gint rows)
  * Public functions
  ***********************************/
 
-GtkType gnome_cmd_string_dialog_get_type ()
-{
-    static GtkType dlg_type = 0;
-
-    if (dlg_type == 0)
-    {
-        GtkTypeInfo dlg_info =
-        {
-            (gchar*) "GnomeCmdStringDialog",
-            sizeof (GnomeCmdStringDialog),
-            sizeof (GnomeCmdStringDialogClass),
-            (GtkClassInitFunc) class_init,
-            (GtkObjectInitFunc) init,
-            /* reserved_1 */ NULL,
-            /* reserved_2 */ NULL,
-            (GtkClassInitFunc) NULL
-        };
-
-        dlg_type = gtk_type_unique (GNOME_CMD_TYPE_DIALOG, &dlg_info);
-    }
-    return dlg_type;
-}
-
-
-GtkWidget *gnome_cmd_string_dialog_new_with_cancel (const gchar *title, const gchar **labels, gint rows, GnomeCmdStringDialogCallback ok_cb, GtkSignalFunc cancel_cb, gpointer user_data)
+GtkWidget *gnome_cmd_string_dialog_new_with_cancel (const gchar *title, const gchar **labels, gint rows, GnomeCmdStringDialogCallback ok_cb, GnomeCmdCallback<GtkButton*> cancel_cb, gpointer user_data)
 {
     auto *dialog = static_cast< GnomeCmdStringDialog* >(g_object_new (GNOME_CMD_TYPE_STRING_DIALOG, NULL));
 
@@ -185,7 +157,7 @@ GtkWidget *gnome_cmd_string_dialog_new_with_cancel (const gchar *title, const gc
 }
 
 
-void gnome_cmd_string_dialog_setup_with_cancel (GnomeCmdStringDialog *dialog, const gchar *title, const gchar **labels, gint rows, GnomeCmdStringDialogCallback ok_cb, GtkSignalFunc cancel_cb, gpointer user_data)
+void gnome_cmd_string_dialog_setup_with_cancel (GnomeCmdStringDialog *dialog, const gchar *title, const gchar **labels, gint rows, GnomeCmdStringDialogCallback ok_cb, GnomeCmdCallback<GtkButton*> cancel_cb, gpointer user_data)
 {
     g_return_if_fail (GNOME_CMD_IS_STRING_DIALOG (dialog));
     g_return_if_fail (title != NULL);
@@ -226,8 +198,9 @@ void gnome_cmd_string_dialog_set_label (GnomeCmdStringDialog *dialog, gint row,
 void gnome_cmd_string_dialog_set_userdata (GnomeCmdStringDialog *dialog, gpointer user_data)
 {
     g_return_if_fail (GNOME_CMD_IS_STRING_DIALOG (dialog));
+    auto priv = static_cast<GnomeCmdStringDialogPrivate*> (gnome_cmd_string_dialog_get_instance_private (dialog));
 
-    dialog->priv->data = user_data;
+    priv->data = user_data;
 }
 
 
@@ -235,16 +208,18 @@ void gnome_cmd_string_dialog_set_ok_cb (GnomeCmdStringDialog *dialog, GnomeCmdSt
 {
     g_return_if_fail (GNOME_CMD_IS_STRING_DIALOG (dialog));
     g_return_if_fail (ok_cb != NULL);
+    auto priv = static_cast<GnomeCmdStringDialogPrivate*> (gnome_cmd_string_dialog_get_instance_private (dialog));
 
-    dialog->priv->ok_cb = ok_cb;
+    priv->ok_cb = ok_cb;
 }
 
 
-void gnome_cmd_string_dialog_set_cancel_cb (GnomeCmdStringDialog *dialog, GtkSignalFunc cancel_cb)
+void gnome_cmd_string_dialog_set_cancel_cb (GnomeCmdStringDialog *dialog, GnomeCmdCallback<GtkButton*> cancel_cb)
 {
     g_return_if_fail (GNOME_CMD_IS_STRING_DIALOG (dialog));
+    auto priv = static_cast<GnomeCmdStringDialogPrivate*> (gnome_cmd_string_dialog_get_instance_private (dialog));
 
-    dialog->priv->cancel_cb = (GFunc)cancel_cb;
+    priv->cancel_cb = cancel_cb;
 }
 
 
@@ -253,7 +228,7 @@ void gnome_cmd_string_dialog_set_value (GnomeCmdStringDialog *dialog, gint row,
     g_return_if_fail (GNOME_CMD_IS_STRING_DIALOG (dialog));
     g_return_if_fail (row >= 0 && row < dialog->rows);
 
-    gtk_entry_set_text (GTK_ENTRY (dialog->entries[row]), value?value:"");
+    gtk_editable_set_text (GTK_EDITABLE (dialog->entries[row]), value ? value : "");
 }
 
 
@@ -261,8 +236,9 @@ void gnome_cmd_string_dialog_set_error_desc (GnomeCmdStringDialog *dialog, gchar
 {
     g_return_if_fail (GNOME_CMD_IS_STRING_DIALOG (dialog));
     g_return_if_fail (msg != NULL);
+    auto priv = static_cast<GnomeCmdStringDialogPrivate*> (gnome_cmd_string_dialog_get_instance_private (dialog));
 
-    g_free (dialog->priv->error_desc);
+    g_free (priv->error_desc);
 
-    dialog->priv->error_desc = g_strdup (msg);
+    priv->error_desc = g_strdup (msg);
 }
