@@ -23,13 +23,7 @@
 
 GtkWidget *lookup_widget (GtkWidget *widget, const gchar *widget_name);
 
-GtkWidget *create_frame (GtkWidget *parent, const gchar *text, gint spacing);
-
-GtkWidget *create_tabframe (GtkWidget *parent);
-
-GtkWidget *create_space_frame (GtkWidget *parent, gint space);
-
-GtkWidget *create_table (GtkWidget *parent, gint rows, gint cols);
+GtkWidget *create_grid (GtkWidget *parent);
 
 GtkWidget *create_vbox (GtkWidget *parent, gboolean h, gint s);
 
@@ -53,8 +47,6 @@ GtkWidget *create_bold_label (GtkWidget *parent, const gchar *text);
 
 GtkWidget *create_hsep (GtkWidget *parent);
 
-GtkWidget *create_space_hbox (GtkWidget *parent, GtkWidget *content);
-
 GtkWidget *create_category (GtkWidget *parent, GtkWidget *content, const gchar *title);
 
 GtkWidget *create_named_button_with_data (GtkWidget *parent, const gchar *label, const gchar *name, GCallback func, gpointer data);
@@ -74,23 +66,6 @@ inline GtkWidget *create_named_button (GtkWidget *parent, const gchar *label, co
     return create_named_button_with_data (parent, label, name, func, parent);
 }
 
-GtkWidget *create_named_stock_button_with_data (GtkWidget *parent, gconstpointer stock, const gchar *name, GCallback func, gpointer data);
-
-inline GtkWidget *create_stock_button_with_data (GtkWidget *parent, gconstpointer stock, GCallback func, gpointer data)
-{
-    return create_named_stock_button_with_data (parent, stock, "button", func, data);
-}
-
-inline GtkWidget *create_named_stock_button (GtkWidget *parent, gconstpointer stock, const gchar *name, GCallback func)
-{
-    return create_named_stock_button_with_data (parent, stock, name, func, parent);
-}
-
-inline GtkWidget *create_stock_button (GtkWidget *parent, gconstpointer stock, GCallback func)
-{
-    return create_stock_button_with_data (parent, stock, func, parent);
-}
-
 GtkWidget *create_entry (GtkWidget *parent, const gchar *name, const gchar *value);
 
 GtkWidget *create_check (GtkWidget *parent, const gchar *text, const gchar *name);
@@ -109,11 +84,11 @@ GtkWidget *create_icon_button_widget (GtkWidget *parent, const gchar *name, cons
 
 GtkWidget *create_scale (GtkWidget *parent, const gchar *name, gint value, gint min, gint max);
 
-GtkWidget *create_directory_chooser_button (GtkWidget *parent, const gchar *name, const gchar *value);
+GtkWidget *create_directory_chooser_button (GtkWidget *parent, const gchar *name, bool local_only = true);
+GFile *directory_chooser_button_get_file (GtkWidget *button);
+void directory_chooser_button_set_file (GtkWidget *button, GFile *file);
 
-GtkWidget *create_file_chooser_button (GtkWidget *parent, const gchar *name, const gchar *value);
-
-GtkWidget *create_treeview (GtkWidget *parent, const gchar *name, GtkTreeModel *model, gint rowh, GtkSignalFunc on_selection_changed, GtkSignalFunc on_rows_reordered);
+GtkWidget *create_treeview (GtkWidget *parent, const gchar *name, GtkTreeModel *model, gint rowh, GCallback on_selection_changed, GCallback on_rows_reordered);
 
 void create_treeview_column (GtkWidget *sw, gint col, gint width, const gchar *label);
 
@@ -125,46 +100,17 @@ GtkWidget *create_combo_box_text_with_entry (GtkWidget *parent);
 
 GtkWidget *create_combo_box_text (GtkWidget *parent, const gchar **items);
 
-inline const gchar *get_combo_box_entry_text (GtkWidget *combo)
-{
-    return gtk_entry_get_text (GTK_ENTRY (gtk_bin_get_child (GTK_BIN (combo))));
-}
-
 inline GSList *get_radio_group (GtkWidget *radio)
 {
     return gtk_radio_button_get_group (GTK_RADIO_BUTTON (radio));
 }
 
-/**
- * Wrapper function for <a href="https://developer.gnome.org/gtk3/stable/GtkTable.html#gtk-table-attach">gtk_table_attach</a> to easily add a widget to a GtkTable table.
- * \param *table A table widget.
- * \param *w The widget which should be added to the table.
- * \param x The column number to attach the left side of a child widget to.
- * \param y The row number to attach the top of a child widget to.
- * \param x_opts Used to specify the properties of the child widget when the table is resized.
- */
-inline void table_add (GtkWidget *table, GtkWidget *w, gint x, gint y, GtkAttachOptions x_opts)
-{
-    gtk_table_attach (GTK_TABLE (table), w, x, x+1, y, y+1, x_opts, (GtkAttachOptions)0, 0, 0);
-}
-
-inline void table_add_y (GtkWidget *table, GtkWidget *w, gint x, gint y, GtkAttachOptions x_opts, GtkAttachOptions y_opts)
-{
-    gtk_table_attach (GTK_TABLE (table), w, x, x+1, y, y+1, x_opts, y_opts, 0, 0);
-}
-
 GtkWidget *create_progress_bar (GtkWidget *parent);
 
 GtkWidget *create_sw (GtkWidget *parent);
 
 const char *get_entry_text (GtkWidget *parent, const gchar *entry_name);
 
-#ifdef __GNUC__
-    void create_error_dialog (const gchar *msg, ...) __attribute__ ((format (gnu_printf, 1, 2)));
-#else
-    void create_error_dialog (const gchar *msg, ...);
-#endif
-
 #if !GTK_CHECK_VERSION(3,0,0)
 inline void gtk_progress_bar_set_show_text (GtkProgressBar *pbar, gboolean show_text)
 {
@@ -187,3 +133,94 @@ inline gboolean gtk_tree_model_iter_previous (GtkTreeModel *tree_model, GtkTreeI
     return result;
 }
 #endif
+
+
+class MenuBuilder
+{
+public:
+    MenuBuilder()
+        : menu(g_menu_new()),
+          accel_group(gtk_accel_group_new()),
+          parent(nullptr),
+          label(nullptr),
+          action_group(nullptr),
+          action_widget(nullptr)
+    {
+    }
+
+    MenuBuilder with_action_group(GActionGroup *action_group)
+    {
+        this->action_group = action_group;
+        return *this;
+    }
+
+    MenuBuilder item(const gchar *label,
+                     const gchar *detailed_action,
+                     const gchar *accelerator = nullptr,
+                     const gchar *icon = nullptr) &&;
+
+    MenuBuilder item(GMenuItem *item) &&;
+
+    MenuBuilder submenu(const gchar *label) &&
+    {
+        return MenuBuilder(this, label);
+    }
+
+    MenuBuilder endsubmenu() &&
+    {
+        g_menu_append_submenu (parent->menu, label, G_MENU_MODEL (menu));
+        return *parent;
+    }
+
+    MenuBuilder section() &&
+    {
+        return MenuBuilder(this, nullptr);
+    }
+
+    MenuBuilder endsection() &&
+    {
+        g_menu_append_section (parent->menu, nullptr, G_MENU_MODEL (menu));
+        return *parent;
+    }
+
+    MenuBuilder section(GMenuModel *section) &&
+    {
+        g_menu_append_section (menu, nullptr, section);
+        return *this;
+    }
+
+    MenuBuilder section(GMenu *section) &&
+    {
+        g_menu_append_section (menu, nullptr, G_MENU_MODEL (section));
+        return *this;
+    }
+
+    struct Result
+    {
+        GMenu *menu;
+        GtkAccelGroup *accel_group;
+    };
+
+    Result build() &&
+    {
+        return Result { menu, accel_group };
+    }
+private:
+    GMenu *menu;
+    GtkAccelGroup *accel_group;
+    MenuBuilder *parent = nullptr;
+    const gchar *label = nullptr;
+    GActionGroup *action_group;
+    GtkWidget *action_widget;
+
+    MenuBuilder(MenuBuilder *parent, const gchar *label = nullptr)
+        : menu(g_menu_new()),
+          accel_group(parent->accel_group),
+          parent(parent),
+          label(label),
+          action_group(parent->action_group),
+          action_widget(parent->action_widget)
+    {
+    }
+};
+
