@@ -31,13 +31,9 @@
 #define GNOME_CMD_MAKE_COPY_DIALOG_GET_CLASS(obj)    (G_TYPE_INSTANCE_GET_CLASS((obj), GNOME_CMD_TYPE_MAKE_COPY_DIALOG, GnomeCmdMakeCopyDialogClass))
 
 
-struct GnomeCmdMakeCopyDialogPrivate;
-
-
 struct GnomeCmdMakeCopyDialog
 {
     GnomeCmdStringDialog parent;
-    GnomeCmdMakeCopyDialogPrivate *priv;
 };
 
 
@@ -49,4 +45,4 @@ struct GnomeCmdMakeCopyDialogClass
 
 GtkWidget *gnome_cmd_make_copy_dialog_new (GnomeCmdFile *f, GnomeCmdDir *dir);
 
-GtkType gnome_cmd_make_copy_dialog_get_type ();
+GType gnome_cmd_make_copy_dialog_get_type ();
