@@ -31,19 +31,15 @@
 #define GNOME_CMD_CHMOD_COMPONENT_GET_CLASS(obj)    (G_TYPE_INSTANCE_GET_CLASS((obj), GNOME_CMD_TYPE_CHMOD_COMPONENT, GnomeCmdChmodComponentClass))
 
 
-struct GnomeCmdChmodComponentPrivate;
-
-
 struct GnomeCmdChmodComponent
 {
-    GtkVBox parent;
-    GnomeCmdChmodComponentPrivate *priv;
+    GtkBox parent;
 };
 
 
 struct GnomeCmdChmodComponentClass
 {
-    GtkVBoxClass parent_class;
+    GtkBoxClass parent_class;
 
     void (* perms_changed)      (GnomeCmdChmodComponent *component);
 };
@@ -51,7 +47,7 @@ struct GnomeCmdChmodComponentClass
 
 GtkWidget *gnome_cmd_chmod_component_new (guint32 perms);
 
-GtkType gnome_cmd_chmod_component_get_type ();
+GType gnome_cmd_chmod_component_get_type ();
 
 guint32 gnome_cmd_chmod_component_get_perms (GnomeCmdChmodComponent *component);
 void gnome_cmd_chmod_component_set_perms (GnomeCmdChmodComponent *component, guint32 permissions);
