@@ -43,6 +43,6 @@ struct GnomeCmdConHomeClass
 };
 
 
-GtkType gnome_cmd_con_home_get_type ();
+GType gnome_cmd_con_home_get_type ();
 
 GnomeCmdCon *gnome_cmd_con_home_new ();
