@@ -50,9 +50,6 @@ struct GnomeCmdConList::Private
     GList *all_cons;
 };
 
-
-static GtkObjectClass *parent_class = nullptr;
-
 enum
 {
     LIST_CHANGED,
@@ -65,18 +62,21 @@ enum
 static guint signals[LAST_SIGNAL] = { 0 };
 
 
+G_DEFINE_TYPE (GnomeCmdConList, gnome_cmd_con_list, G_TYPE_OBJECT)
+
+
 static void on_con_updated (GnomeCmdCon *con, GnomeCmdConList *con_list)
 {
     g_return_if_fail (GNOME_CMD_IS_CON (con));
     g_return_if_fail (GNOME_CMD_IS_CON_LIST (con_list));
 
     if (GNOME_CMD_IS_CON_REMOTE (con))
-        gtk_signal_emit (*con_list, signals[REMOTE_LIST_CHANGED]);
+        g_signal_emit (con_list, signals[REMOTE_LIST_CHANGED], 0);
     else
         if (GNOME_CMD_IS_CON_DEVICE (con))
-            gtk_signal_emit (*con_list, signals[DEVICE_LIST_CHANGED]);
+            g_signal_emit (con_list, signals[DEVICE_LIST_CHANGED], 0);
 
-    gtk_signal_emit (*con_list, signals[LIST_CHANGED]);
+    g_signal_emit (con_list, signals[LIST_CHANGED], 0);
 }
 
 
@@ -90,59 +90,59 @@ static gint compare_alias (const GnomeCmdCon *c1, const GnomeCmdCon *c2)
  * Gtk class implementation
  *******************************/
 
-static void destroy (GtkObject *object)
+static void dispose (GObject *object)
 {
     GnomeCmdConList *con_list = GNOME_CMD_CON_LIST (object);
 
-    g_free (con_list->priv);
+    g_clear_pointer (&con_list->priv, g_free);
 
-    if (GTK_OBJECT_CLASS (parent_class)->destroy)
-        (*GTK_OBJECT_CLASS (parent_class)->destroy) (object);
+    G_OBJECT_CLASS (gnome_cmd_con_list_parent_class)->dispose (object);
 }
 
 
-static void class_init (GnomeCmdConListClass *klass)
+static void gnome_cmd_con_list_class_init (GnomeCmdConListClass *klass)
 {
-    GtkObjectClass *object_class = GTK_OBJECT_CLASS (klass);
-    parent_class = (GtkObjectClass *) gtk_type_class (gtk_object_get_type ());
+    signals[LIST_CHANGED]           = g_signal_new ("list-changed",
+                                                    G_TYPE_FROM_CLASS (klass),
+                                                    G_SIGNAL_RUN_LAST,
+                                                    G_STRUCT_OFFSET (GnomeCmdConListClass, list_changed),
+                                                    nullptr, nullptr,
+                                                    g_cclosure_marshal_VOID__VOID,
+                                                    G_TYPE_NONE,
+                                                    0);
 
-    signals[LIST_CHANGED]           = gtk_signal_new ("list-changed",
-                                                              GTK_RUN_LAST,
-                                                              G_OBJECT_CLASS_TYPE (object_class),
-                                                              GTK_SIGNAL_OFFSET (GnomeCmdConListClass, list_changed),
-                                                              gtk_marshal_NONE__NONE,
-                                                              GTK_TYPE_NONE,
-                                                              0);
+    signals[REMOTE_LIST_CHANGED]    = g_signal_new ("remote-list-changed",
+                                                    G_TYPE_FROM_CLASS (klass),
+                                                    G_SIGNAL_RUN_LAST,
+                                                    G_STRUCT_OFFSET (GnomeCmdConListClass, ftp_list_changed),
+                                                    nullptr, nullptr,
+                                                    g_cclosure_marshal_VOID__VOID,
+                                                    G_TYPE_NONE,
+                                                    0);
 
-    signals[REMOTE_LIST_CHANGED]    = gtk_signal_new ("remote-list-changed",
-                                                               GTK_RUN_LAST,
-                                                               G_OBJECT_CLASS_TYPE (object_class),
-                                                               GTK_SIGNAL_OFFSET (GnomeCmdConListClass, ftp_list_changed),
-                                                               gtk_marshal_NONE__NONE,
-                                                               GTK_TYPE_NONE,
-                                                               0);
+    signals[DEVICE_LIST_CHANGED]    = g_signal_new ("device-list-changed",
+                                                    G_TYPE_FROM_CLASS (klass),
+                                                    G_SIGNAL_RUN_LAST,
+                                                    G_STRUCT_OFFSET (GnomeCmdConListClass, device_list_changed),
+                                                    nullptr, nullptr,
+                                                    g_cclosure_marshal_VOID__VOID,
+                                                    G_TYPE_NONE,
+                                                    0);
 
-    signals[DEVICE_LIST_CHANGED]    = gtk_signal_new ("device-list-changed",
-                                                               GTK_RUN_LAST,
-                                                               G_OBJECT_CLASS_TYPE (object_class),
-                                                               GTK_SIGNAL_OFFSET (GnomeCmdConListClass, device_list_changed),
-                                                               gtk_marshal_NONE__NONE,
-                                                               GTK_TYPE_NONE,
-                                                               0);
+    signals[QUICK_FTP_LIST_CHANGED] = g_signal_new ("quick-ftp-list-changed",
+                                                    G_TYPE_FROM_CLASS (klass),
+                                                    G_SIGNAL_RUN_LAST,
+                                                    G_STRUCT_OFFSET (GnomeCmdConListClass, quick_ftp_list_changed),
+                                                    nullptr, nullptr,
+                                                    g_cclosure_marshal_VOID__VOID,
+                                                    G_TYPE_NONE,
+                                                    0);
 
-    signals[QUICK_FTP_LIST_CHANGED] = gtk_signal_new ("quick-ftp-list-changed",
-                                                               GTK_RUN_LAST,
-                                                               G_OBJECT_CLASS_TYPE (object_class),
-                                                               GTK_SIGNAL_OFFSET (GnomeCmdConListClass, quick_ftp_list_changed),
-                                                               gtk_marshal_NONE__NONE,
-                                                               GTK_TYPE_NONE,
-                                                               0);
-
-    object_class->destroy = destroy;
+    G_OBJECT_CLASS (klass)->dispose = dispose;
 }
 
 
-static void init (GnomeCmdConList *con_list)
+static void gnome_cmd_con_list_init (GnomeCmdConList *con_list)
 {
     con_list->priv = g_new0 (GnomeCmdConList::Private, 1);
 
@@ -163,36 +163,10 @@ static void init (GnomeCmdConList *con_list)
 #endif
 }
 
-
-
 /***********************************
  * Public functions
  ***********************************/
 
-GtkType gnome_cmd_con_list_get_type ()
-{
-    static GtkType type = 0;
-
-    if (type == 0)
-    {
-        GtkTypeInfo info =
-        {
-            (gchar*) "GnomeCmdConList",
-            sizeof (GnomeCmdConList),
-            sizeof (GnomeCmdConListClass),
-            (GtkClassInitFunc) class_init,
-            (GtkObjectInitFunc) init,
-            /* reserved_1 */ nullptr,
-            /* reserved_2 */ nullptr,
-            (GtkClassInitFunc) nullptr
-        };
-
-        type = gtk_type_unique (gtk_object_get_type (), &info);
-    }
-    return type;
-}
-
-
 void GnomeCmdConList::lock()
 {
     priv->update_lock = TRUE;
@@ -205,11 +179,11 @@ void GnomeCmdConList::lock()
 void GnomeCmdConList::unlock()
 {
     if (priv->changed)
-        gtk_signal_emit (*this, signals[LIST_CHANGED]);
+        g_signal_emit (this, signals[LIST_CHANGED], 0);
     if (priv->remote_cons_changed)
-        gtk_signal_emit (*this, signals[REMOTE_LIST_CHANGED]);
+        g_signal_emit (this, signals[REMOTE_LIST_CHANGED], 0);
     if (priv->device_cons_changed)
-        gtk_signal_emit (*this, signals[DEVICE_LIST_CHANGED]);
+        g_signal_emit (this, signals[DEVICE_LIST_CHANGED], 0);
 
     priv->update_lock = FALSE;
 }
@@ -232,8 +206,8 @@ void GnomeCmdConList::add(GnomeCmdConRemote *con)
     }
     else
     {
-        gtk_signal_emit (*this, signals[LIST_CHANGED]);
-        gtk_signal_emit (*this, signals[REMOTE_LIST_CHANGED]);
+        g_signal_emit (this, signals[LIST_CHANGED], 0);
+        g_signal_emit (this, signals[REMOTE_LIST_CHANGED], 0);
     }
 }
 
@@ -255,8 +229,8 @@ void GnomeCmdConList::remove(GnomeCmdConRemote *con)
     }
     else
     {
-        gtk_signal_emit (*this, signals[LIST_CHANGED]);
-        gtk_signal_emit (*this, signals[REMOTE_LIST_CHANGED]);
+        g_signal_emit (this, signals[LIST_CHANGED], 0);
+        g_signal_emit (this, signals[REMOTE_LIST_CHANGED], 0);
     }
 }
 
@@ -279,8 +253,8 @@ void gnome_cmd_con_list_add_quick_ftp (GnomeCmdConList *con_list, GnomeCmdConRem
     }
     else
     {
-        gtk_signal_emit (*con_list, signals[LIST_CHANGED]);
-        gtk_signal_emit (*con_list, signals[QUICK_FTP_LIST_CHANGED]);
+        g_signal_emit (con_list, signals[LIST_CHANGED], 0);
+        g_signal_emit (con_list, signals[QUICK_FTP_LIST_CHANGED], 0);
     }
 }
 
@@ -301,8 +275,8 @@ void GnomeCmdConList::add(GnomeCmdConDevice *con)
     }
     else
     {
-        gtk_signal_emit (*this, signals[LIST_CHANGED]);
-        gtk_signal_emit (*this, signals[DEVICE_LIST_CHANGED]);
+        g_signal_emit (this, signals[LIST_CHANGED], 0);
+        g_signal_emit (this, signals[DEVICE_LIST_CHANGED], 0);
     }
 }
 
@@ -323,8 +297,8 @@ void GnomeCmdConList::remove(GnomeCmdConDevice *con)
     }
     else
     {
-        gtk_signal_emit (*this, signals[LIST_CHANGED]);
-        gtk_signal_emit (*this, signals[DEVICE_LIST_CHANGED]);
+        g_signal_emit (this, signals[LIST_CHANGED], 0);
+        g_signal_emit (this, signals[DEVICE_LIST_CHANGED], 0);
     }
 }
 
@@ -361,6 +335,21 @@ void gnome_cmd_con_list_set_all_dev (GnomeCmdConList *con_list, GList *dev_cons)
 }
 
 
+GnomeCmdCon *gnome_cmd_con_list_find_by_uuid (GnomeCmdConList *con_list, const gchar *uuid)
+{
+    g_return_val_if_fail (GNOME_CMD_IS_CON_LIST (con_list), nullptr);
+    g_return_val_if_fail (uuid != nullptr, nullptr);
+
+    for (GList *elem = con_list->priv->all_cons; elem; elem = elem->next)
+    {
+        auto con = static_cast<GnomeCmdCon*> (elem->data);
+        if (!strcmp (gnome_cmd_con_get_uuid (con), uuid))
+            return con;
+    }
+    return nullptr;
+}
+
+
 GnomeCmdCon *GnomeCmdConList::find_alias(const gchar *alias) const
 {
     g_return_val_if_fail (alias != nullptr, nullptr);
@@ -406,4 +395,4 @@ GnomeCmdCon *get_remote_con_for_gfile(GFile *gFile)
         g_free(gFileSrcUri);
     }
     return gnomeCmdCon;
-}
\ No newline at end of file
+}
