@@ -37,7 +37,7 @@ struct GnomeCmdConSmbClass
 };
 
 
-static GnomeCmdConClass *parent_class = nullptr;
+G_DEFINE_TYPE (GnomeCmdConSmb, gnome_cmd_con_smb, GNOME_CMD_TYPE_CON)
 
 
 static void mount_func (GnomeCmdCon *con)
@@ -170,47 +170,57 @@ static GnomeCmdPath *smb_create_path (GnomeCmdCon *con, const gchar *path_str)
 }
 
 
+static gchar *smb_get_go_text (GnomeCmdCon *con)
+{
+    return g_strdup (_("Go to: Samba Network"));
+}
+
+
+static gchar *smb_get_open_text (GnomeCmdCon *con)
+{
+    return nullptr;
+}
+
+
+static gchar *smb_get_close_text (GnomeCmdCon *con)
+{
+    return nullptr;
+}
+
+
+static GIcon *smb_get_icon (GnomeCmdCon *con)
+{
+    return g_themed_icon_new ("folder-remote");
+}
+
 
 /*******************************
  * Gtk class implementation
  *******************************/
 
-static void destroy (GtkObject *object)
+static void gnome_cmd_con_smb_class_init (GnomeCmdConSmbClass *klass)
 {
-    GnomeCmdConSmb *con_smb = GNOME_CMD_CON_SMB (object);
-
-    gnome_cmd_pixmap_free (con_smb->parent.go_pixmap);
-    gnome_cmd_pixmap_free (con_smb->parent.open_pixmap);
-    gnome_cmd_pixmap_free (con_smb->parent.close_pixmap);
-    g_free(con_smb->parent.uri);
-
-    if (GTK_OBJECT_CLASS (parent_class)->destroy)
-        (*GTK_OBJECT_CLASS (parent_class)->destroy) (object);
-}
-
-
-static void class_init (GnomeCmdConSmbClass *klass)
-{
-    GtkObjectClass *object_class = GTK_OBJECT_CLASS (klass);
     GnomeCmdConClass *con_class = GNOME_CMD_CON_CLASS (klass);
 
-    parent_class = static_cast<GnomeCmdConClass*> (gtk_type_class (GNOME_CMD_TYPE_CON));
-
-    object_class->destroy = destroy;
-
     con_class->open = smb_open;
     con_class->close = smb_close;
     con_class->cancel_open = smb_cancel_open;
     con_class->open_is_needed = smb_open_is_needed;
     con_class->create_gfile = smb_create_gfile;
     con_class->create_path = smb_create_path;
+
+    con_class->get_go_text = smb_get_go_text;
+    con_class->get_open_text = smb_get_open_text;
+    con_class->get_close_text = smb_get_close_text;
+
+    con_class->get_go_icon = smb_get_icon;
+    con_class->get_open_icon = smb_get_icon;
+    con_class->get_close_icon = smb_get_icon;
 }
 
 
-static void init (GnomeCmdConSmb *smb_con)
+static void gnome_cmd_con_smb_init (GnomeCmdConSmb *smb_con)
 {
-    guint dev_icon_size = gnome_cmd_data.dev_icon_size;
-
     GnomeCmdCon *con = GNOME_CMD_CON (smb_con);
 
     con->alias = g_strdup (_("SMB"));
@@ -222,37 +232,5 @@ static void init (GnomeCmdConSmb *smb_con)
     con->can_show_free_space = FALSE;
     con->is_local = FALSE;
     con->is_closeable = FALSE;
-    con->go_text = g_strdup (_("Go to: Samba Network"));
-    con->go_pixmap = gnome_cmd_pixmap_new_from_icon ("folder-remote", dev_icon_size);
-    con->open_pixmap = gnome_cmd_pixmap_new_from_icon ("folder-remote", dev_icon_size);
-    con->close_pixmap = gnome_cmd_pixmap_new_from_icon ("folder-remote", dev_icon_size);
 }
 
-
-
-/***********************************
- * Public functions
- ***********************************/
-
-GtkType gnome_cmd_con_smb_get_type ()
-{
-    static GtkType type = 0;
-
-    if (type == 0)
-    {
-        GtkTypeInfo info =
-        {
-            (gchar*) "GnomeCmdConSmb",
-            sizeof (GnomeCmdConSmb),
-            sizeof (GnomeCmdConSmbClass),
-            (GtkClassInitFunc) class_init,
-            (GtkObjectInitFunc) init,
-            /* reserved_1 */ nullptr,
-            /* reserved_2 */ nullptr,
-            (GtkClassInitFunc) nullptr
-        };
-
-        type = gtk_type_unique (GNOME_CMD_TYPE_CON, &info);
-    }
-    return type;
-}
