@@ -42,11 +42,13 @@ enum COPY_ERROR_ACTION
 
 struct XferData
 {
+    GtkWindow *parent_window;
+
     GFileCopyFlags copyFlags;
     GnomeCmdConfirmOverwriteMode overwriteMode;
     GnomeCmdTransferType transferType{COPY};
 
-    // Source and target GFile's. The first srcGFile should be transfered to the first destGFile and so on...
+    // Source and target GFile's. The first srcGFile should be transferred to the first destGFile and so on...
     GList *srcGFileList;
     GList *destGFileList;
 
@@ -80,7 +82,7 @@ struct XferData
     GFile *problemDestGFile;
     GThread *thread{nullptr};                // the work thread
     GMutex mutex{nullptr};                   // used to sync the main and worker thread
-    GError *error{nullptr};                  // the cause that the file cant be deleted
+    GError *error{nullptr};                  // the cause that the file can't be deleted
     GType currentSrcFileType;                // the file type of the file which is currently copied
 };
 
@@ -91,7 +93,7 @@ gnome_cmd_copy_start (GList *src_files,
                       gchar *dest_fn,
                       GFileCopyFlags copyFlags,
                       GnomeCmdConfirmOverwriteMode overwriteMode,
-                      GtkSignalFunc on_completed_func,
+                      GCallback on_completed_func,
                       gpointer on_completed_data);
 
 void
@@ -101,7 +103,7 @@ gnome_cmd_move_start (GList *src_files,
                       gchar *dest_fn,
                       GFileCopyFlags copyFlags,
                       GnomeCmdConfirmOverwriteMode overwriteMode,
-                      GtkSignalFunc on_completed_func,
+                      GCallback on_completed_func,
                       gpointer on_completed_data);
 
 void
@@ -112,7 +114,7 @@ gnome_cmd_copy_gfiles_start (GList *src_uri_list,
                            gchar *dest_fn,
                            GFileCopyFlags copyFlags,
                            GnomeCmdConfirmOverwriteMode overwriteMode,
-                           GtkSignalFunc on_completed_func,
+                           GCallback on_completed_func,
                            gpointer on_completed_data);
 
 void
@@ -123,7 +125,7 @@ gnome_cmd_move_gfiles_start (GList *src_uri_list,
                            gchar *dest_fn,
                            GFileCopyFlags copyFlags,
                            GnomeCmdConfirmOverwriteMode overwriteMode,
-                           GtkSignalFunc on_completed_func,
+                           GCallback on_completed_func,
                            gpointer on_completed_data);
 
 void
@@ -134,14 +136,15 @@ gnome_cmd_link_gfiles_start (GList *src_uri_list,
                            gchar *dest_fn,
                            GFileCopyFlags copyFlags,
                            GnomeCmdConfirmOverwriteMode overwriteMode,
-                           GtkSignalFunc on_completed_func,
+                           GCallback on_completed_func,
                            gpointer on_completed_data);
 
 void
-gnome_cmd_tmp_download (GList *srcGFileList,
+gnome_cmd_tmp_download (GtkWindow *parent_window,
+                        GList *srcGFileList,
                         GList *destGFileList,
                         GFileCopyFlags copyFlags,
-                        GtkSignalFunc on_completed_func,
+                        GCallback on_completed_func,
                         gpointer on_completed_data);
 
 void
