@@ -45,29 +45,29 @@ static LsColor *type_colors[7];
 # Background color codes:
 # 40=black 41=red 42=green 43=yellow 44=blue 45=magenta 46=cyan 47=white
 */
-inline GdkColor *code2color (gint code)
+inline GdkRGBA *code2color (gint code)
 {
     GnomeCmdLsColorsPalette &palette = gnome_cmd_data.options.ls_colors_palette;
 
     switch (code)
     {
-        case 30: return palette.black_fg;
-        case 31: return palette.red_fg;
-        case 32: return palette.green_fg;
-        case 33: return palette.yellow_fg;
-        case 34: return palette.blue_fg;
-        case 35: return palette.magenta_fg;
-        case 36: return palette.cyan_fg;
-        case 37: return palette.white_fg;
+        case 30: return &palette.black_fg;
+        case 31: return &palette.red_fg;
+        case 32: return &palette.green_fg;
+        case 33: return &palette.yellow_fg;
+        case 34: return &palette.blue_fg;
+        case 35: return &palette.magenta_fg;
+        case 36: return &palette.cyan_fg;
+        case 37: return &palette.white_fg;
 
-        case 40: return palette.black_bg;
-        case 41: return palette.red_bg;
-        case 42: return palette.green_bg;
-        case 43: return palette.yellow_bg;
-        case 44: return palette.blue_bg;
-        case 45: return palette.magenta_bg;
-        case 46: return palette.cyan_bg;
-        case 47: return palette.white_bg;
+        case 40: return &palette.black_bg;
+        case 41: return &palette.red_bg;
+        case 42: return &palette.green_bg;
+        case 43: return &palette.yellow_bg;
+        case 44: return &palette.blue_bg;
+        case 45: return &palette.magenta_bg;
+        case 46: return &palette.cyan_bg;
+        case 47: return &palette.white_bg;
         default: break;
     }
 
