@@ -77,7 +77,7 @@ inline void run_command (const gchar *command)
 
 const char **convert_varargs_to_name_array (va_list args);
 
-gint run_simple_dialog (GtkWidget *parent, gboolean ignore_close_box,
+gint run_simple_dialog (GtkWindow *parent, gboolean ignore_close_box,
                         GtkMessageType msg_type,
                         const char *text, const char *title, gint def_response, ...);
 
@@ -140,7 +140,13 @@ const gchar *perm2numstring (guint32 permissions, gchar *buf, guint max);
 const gchar *size2string (guint64 size, GnomeCmdSizeDispMode size_disp_mode);
 const gchar *time2string (GDateTime *gDateTime, const gchar *date_format);
 
-void clear_event_key (GdkEventKey *event);
+inline GdkModifierType get_modifiers_state()
+{
+    GdkWindow *window = gdk_window_at_pointer (nullptr, nullptr);
+    GdkModifierType mask = (GdkModifierType) 0;
+    gdk_window_get_pointer (window, nullptr, nullptr, &mask);
+    return mask;
+}
 
 inline gboolean state_is_blank (gint state)
 {
@@ -198,7 +204,6 @@ gchar *unquote_if_needed (const gchar *in);
 GList *string_history_add (GList *in, const gchar *value, guint maxsize);
 
 GtkWidget *create_styled_button (const gchar *text);
-GtkWidget *create_styled_pixmap_button (const gchar *text, GnomeCmdPixmap *pixmap);
 
 void set_cursor_busy_for_widget (GtkWidget *widget);
 
@@ -220,7 +225,7 @@ gchar *get_temp_download_filepath (const gchar *fname);
 void remove_temp_download_dir ();
 
 gchar *unix_to_unc (const gchar *path);
-GdkColor *gdk_color_new (gushort r, gushort g, gushort b);
+GdkRGBA *gdk_color_new (gushort r, gushort g, gushort b);
 GList *gnome_cmd_file_list_to_gfile_list (GList *files);
 
 gboolean is_dir_existing(const gchar *dpath);
@@ -254,7 +259,7 @@ inline gint gnome_cmd_prompt_message (GtkWindow *parent, GtkMessageType type, Gt
 
     gint result = gtk_dialog_run (GTK_DIALOG (dlg));
 
-    gtk_widget_destroy (dlg);
+    gtk_window_destroy (GTK_WINDOW (dlg));
 
     return result;
 }
